/***********************************************************************
 * PROJECT NAME       : LTE Software
 * Component name     : Measurement Based Transport Admission Control
 * File name          : mbtac.c
 * Author             : 
 * ---------------------------------------------------------------------
 * Description        :
 *
 ***********************************************************************
 * Copyright (c) Nokia
 * All rights reserved.
 * Licensed material - property of Nokia
 ***********************************************************************/

/** @file mbtac.c */

#ifndef __MBTAC_IWF_SRC_C__
#define __MBTAC_IWF_SRC_C__


#include "ip_secfg_IWF_VTC.h"
#include <linux/time.h>
//Global Variables
uint8_t          mbtac_l2_incl_flag      = 0;
uint64_t         mbtac_dscp_cfg          = MBTAC_DEFAULT_DSCP_CFG; // bits 46,34,28,26 set to one
mbTacDscp_t      mbtac_dscp_table        = { QCI_DEFAULT_LEVELS, {
                                                            MBTAC_DEFAULT_DSCP1 , // 46
                                                            MBTAC_DEFAULT_DSCP2 , // 26
                                                            MBTAC_DEFAULT_DSCP3 , // 46
                                                            MBTAC_DEFAULT_DSCP4 , // 28
                                                            MBTAC_DEFAULT_DSCP5 , // 34
                                                            0, 0, 0,
                                                            0, 0, 0, 0,  0, 0, 0, 0,
                                                            0, 0, 0, 0,  0, 0, 0, 0,
                                                            0, 0
                                                            } }; // default taken from WBTS_TRS_3751
double           mbTacTimeFactor         = 0;
mbTacFpTmbData_t mbtacTmbCounterData;

uint64_t         mbTac_perf_start        = 0;
uint64_t         mbTac_perf_stop         = 0;

/*****************************************************************************
 * Method Name        :mbtac_ipse_api 
 *
 * Input Parameters   : sIpSicpApiMsg , sKernelIpCfgMsgHeader
 *
 * Returns            : VTC_OK / VTC_ERR
 *
 * Description        : MBTAC Configuration API for IWF module
 *
 ****************************************************************************/
uint32_t mbtac_ipse_api (sIpSicpApiMsg *msg,sKernelIpCfgMsgHeader* pMsgHeader )
{
    mbtac_api_t *cfg = (mbtac_api_t *)&msg->mbTacConfig;
    uint32_t retCode = ERR_INVALID_CONFIGURATION;
    switch (cfg->type)
    {
        case MBTAC_GET_FP_TMB_MEASUREMENTS:
            retCode = mbTacGetFpTmbMeasurements(msg);
            pMsgHeader->msgFlags |= ICM_RETURN_WITH_DATA;
            break;

        case MBTAC_SET_DSCP:
            retCode = mbTacSetDscpValues(msg);
            break;

        case MBTAC_GET_DSCP:
            retCode = mbTacGetDscpValues(msg);
            pMsgHeader->msgFlags |= ICM_RETURN_WITH_DATA;
            break;

        case MBTAC_SET_INCLUDE_L2_OVERHEAD:
            retCode = mbTacSetFpL2InclFlag(msg);
            break;

        case MBTAC_GET_INCLUDE_L2_OVERHEAD:
            retCode = mbTacGetFpL2InclFlag(msg);
            pMsgHeader->msgFlags |= ICM_RETURN_WITH_DATA;
            break;

        case MBTAC_GET_DSCP_CONFIGURATION:
            retCode = mbTacGetDscpConfiguration(msg);
            pMsgHeader->msgFlags |= ICM_RETURN_WITH_DATA;
            break;

        default:
            //printk("SE_ERROR: unknown API message\n");
            break;
    }
return retCode;
}


/*****************************************************************************
 * Method Name        :mbTacInit 
 *
 * Input Parameters   : 
 *
 * Returns            : 
 *
 * Description        : To initialize the MBTAC counters /Stats
 *
 ****************************************************************************/
uint32_t mbTacInit ()
{
    memset(&mbtacTmbCounterData, 0, sizeof(mbTacFpTmbData_t));
    return VTC_OK;
}

/*****************************************************************************
 * Method Name        :ReadAndClearCounters
 *
 * Input Parameters   : mbTacFpTmbData_t
 *
 * Returns            : 
 *
 * Description        : To Read the MBTAC counters /Stats from the global structure
 *
 ****************************************************************************/
static inline void ReadAndClearCounters(mbTacFpTmbData_t* dataToReturn)
{
    int      i = 0;
    static int cnt = 0;

    //dataToReturn->elapsedTime = (uint32_t)(mbTacEndTime - mbTacStartTime);
    dataToReturn->elapsedTime = 0;
    memcpy(&dataToReturn->splaneData, &mbtacTmbCounterData.splaneData, sizeof(mbTacFpTmbDataPerIpAddr_t));
    mbtacTmbCounterData.splaneData.bytesSumUl = 0;
    mbtacTmbCounterData.splaneData.bytesSumDl = 0;
    cnt++;
    for(i = 0; i < MAX_NO_CPLANE_IP_ADDR; i++)
    {
        memcpy(&dataToReturn->cplaneData[i], &mbtacTmbCounterData.cplaneData[i], sizeof(mbTacFpTmbDataPerIpAddr_t));
        mbtacTmbCounterData.cplaneData[i].bytesSumUl = 0;
        mbtacTmbCounterData.cplaneData[i].bytesSumDl = 0;
    }
    memcpy(&dataToReturn->mcastcplaneData, &mbtacTmbCounterData.mcastcplaneData, sizeof(mbTacFpTmbDataPerIpAddr_t));
    mbtacTmbCounterData.mcastcplaneData.bytesSumUl = 0;
    mbtacTmbCounterData.mcastcplaneData.bytesSumDl = 0;

}

/*****************************************************************************
 * Method Name        :mbTacGetFpTmbMeasurements
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : 
 *
 * Description        : To Read the MBTAC counters /Stats from the global structure
 *
 ****************************************************************************/
uint32_t mbTacGetFpTmbMeasurements (sIpSicpApiMsg *swp)
{
    uint32_t retVal  = VTC_OK;
    mbtac_api_t         *cfg        = (mbtac_api_t *) &(swp->mbTacConfig);
    mbTacFpTmbData_t    *retData   = (mbTacFpTmbData_t *) &cfg->u.mbTacFpTmbData;

    ReadAndClearCounters(retData);

    return retVal;
}

/*****************************************************************************
 * Method Name        :mbTacSetDscpValues
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : 
 *
 * Description        : To Set the DSCP vlaues for MBTAC traffic
 *
 ****************************************************************************/
uint32_t mbTacSetDscpValues (sIpSicpApiMsg *swp)
{
    uint32_t retVal        = VTC_OK;
    uint64_t temp_dscp_cfg = 0;
    uint8_t  dscp_index    = 0;

    mbtac_api_t *cfg = (mbtac_api_t *) &(swp->mbTacConfig);

    memcpy(&mbtac_dscp_table, (mbTacDscp_t *) &cfg->u.mbTacDscp, sizeof(mbTacDscp_t) );

    for (dscp_index = 0; dscp_index < mbtac_dscp_table.noQciDscpValues; dscp_index++)
    {
        if(mbtac_dscp_table.dscp[dscp_index] < 64)
        {
            temp_dscp_cfg = temp_dscp_cfg | (((uint64_t)0x01) << mbtac_dscp_table.dscp[dscp_index]);
        }
    }
      mbtac_dscp_cfg = temp_dscp_cfg;

    return retVal;
}

/*****************************************************************************
 * Method Name        :mbTacGetDscpValues
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : 
 *
 * Description        : To Get the Configured DSCP vlaues  of MBTAC traffic
 *
 ****************************************************************************/
uint32_t mbTacGetDscpValues (sIpSicpApiMsg *swp)
{
    uint32_t retVal  = VTC_OK;

    mbtac_api_t    *cfg        = (mbtac_api_t *) &(swp->mbTacConfig);
    mbTacDscp_t    *ret_data   = (mbTacDscp_t *) &cfg->u.mbTacDscp;

    memcpy(ret_data, &mbtac_dscp_table, sizeof(mbTacDscp_t));


    return retVal;
}

/*****************************************************************************
 * Method Name        :mbTacSetFpL2InclFlag
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : VTC_OK 
 *
 * Description        : To Set the L2 overhead Flag for MBTAC traffic
 *
 ****************************************************************************/
uint32_t mbTacSetFpL2InclFlag (sIpSicpApiMsg *swp)
{
    uint32_t retVal  = VTC_OK;

    mbtac_api_t *cfg = (mbtac_api_t *) &swp->mbTacConfig;

    mbtac_l2_incl_flag = cfg->u.mbTacL2Flag.l2_incl_flag;
    
    return retVal;
}

/*****************************************************************************
 * Method Name        :mbTacGetFpL2InclFlag
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : VTC_OK 
 *
 * Description        : To Get the L2 overhead Flag for MBTAC traffic
 *
 ****************************************************************************/
uint32_t mbTacGetFpL2InclFlag (sIpSicpApiMsg *swp)
{
    uint32_t retVal  = VTC_OK;

    mbtac_api_t     *cfg        = (mbtac_api_t *) &swp->mbTacConfig;
    mbTacL2Flag_t   *ret_data   = (mbTacL2Flag_t *) &cfg->u.mbTacL2Flag;

    ret_data->l2_incl_flag = mbtac_l2_incl_flag;

    return retVal;
}

/*****************************************************************************
 * Method Name        :mbTacGetDscpConfiguration
 *
 * Input Parameters   : sIpSicpApiMsg
 *
 * Returns            : VTC_OK 
 *
 * Description        : To Get the DSCP Configuration for MBTAC traffic
 *
 ****************************************************************************/
uint32_t mbTacGetDscpConfiguration (sIpSicpApiMsg *swp)
{
    uint32_t     retVal  = VTC_OK;
    mbtac_api_t* cfg     = (mbtac_api_t *) &swp->mbTacConfig;

    cfg->u.mbTacDscpCfg = mbtac_dscp_cfg;

    return retVal;
}


#endif // __MBTAC_IPSE_SRC_C__

